/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Patrick Boenzli
   co-programmer: ...
*/


/* this is for debug output. It just says, that all calls to PRINT() belong to the DEBUG_MODULE_NETWORK module
   For more information refere to https://www.orxonox.net/cgi-bin/trac.cgi/wiki/DebugOutput
*/
#define DEBUG_MODULE_NETWORK


#include "network_stream.h"
#include "list.h"
#include "class_list.h"

#include "debug.h"


/* include your own header */
#include "network_manager.h"


/* using namespace std is default, this needs to be here */
using namespace std;


/************************************
  What you will see here are the function definitions from the header file (network_manager.h) with doxygen documentation. Here is an example:


 In file network_manager.h

 class NetworkManager
 {
   int doSomeStuff(float argument, float* pointer);
 }

 will be implemented in the source file as follows:

 In file network_manager.cc

 / **
  *  this is the short description for this function: it just does some stuff
  * @param argument: this is the first argument, stuff...
  * @param pointer:  this is the pointer to nowhereland
  * return: whatever the function returns: for example an index, number, etc.
  * /
 int NetworkManager::doSomeStuff(float argument, float* pointer)
 {
   // whaterver you want to do
 }


 if you want to make automake compile your files: you will want to add the file names to the local Makefile.am

 ************************************/



/**
 *  standard constructor
 */
NetworkManager::NetworkManager()
{
  /* set the class id for the base object */
  this->setClassID(CL_NETWORK_MANAGER, "NetworkManager");

  /* initialize the references */
  this->netStreamList = NULL;
  this->syncList = NULL;

  PRINTF(0)("NetworkManager created\n");
}


/**
 *  standard deconstructor
 */
NetworkManager::~NetworkManager()
{}


/**
 *  initializes the network manager
 */
void NetworkManager::initialize()
{
  /* get the synchronizeable list from the class list */
  this->netStreamList = ClassList::getList(CL_SYNCHRONIZEABLE);
  PRINTF(0)("NetworkManager initzalized\n");
}


/**
 *  shutsdown the network manager
 */
void NetworkManager::shutdown()
{

}


/**
 *  creates a connection from one object to a host
 * @param hostName: the name of the destination host
 * @param synchronizeable: reference to the sync object
 */
NetworkStream& establishConnection(const char& hostName, const Synchronizeable& sync)
{}


/**
 *  creates a connection from one object to a host
 * @param address: the address of the destination host
 * @param synchronizeable: reference to the sync object
 */
NetworkStream& NetworkManager::establishConnection(IPaddress& address, Synchronizeable& sync)
{
  printf("Establish connection to server %s, on port %u\n", SDLNet_ResolveIP(&address), address.port);
  /* creating a new network stream, it will register itself automaticaly to the class list */
  NetworkStream* netStream = new NetworkStream(address, sync, NET_CLIENT);
}

/**
 *  creates a new NetworkStream of server type
 * @param sync: the listener
 */
NetworkStream& NetworkManager::createServer(Synchronizeable& sync, unsigned int port)
{
  PRINTF(0)("Create a new server socket\n");
  /* creating a new network stream, it will register itself automaticaly to the class list */
  NetworkStream* netStream = new NetworkStream(port, sync, NET_SERVER);
}


/**
 *  teardown a connection
 */
void NetworkManager::shutdownConnection()
{
  PRINTF(0)("Shutdown connection\n");
}



/**
 *  sync the network
 */
void NetworkManager::synchronize()
{
  if (this->netStreamList != NULL || (this->netStreamList = ClassList::getList(CL_NETWORK_STREAM)) != NULL)
  {
    tIterator<BaseObject>* iterator = this->netStreamList->getIterator();
    NetworkStream* stream = (NetworkStream*)(iterator->firstElement());
    while( stream)
    {
      stream->processData();
      stream = (NetworkStream*)(iterator->nextElement());
    }
    delete iterator;
  }

}


